/*

 Receive orders sent by pyrduino module (Python) on serial port. 
 Created by Álvaro Muñoz - alvaromm@correo.ugr.es
 based on Python-Arduino-Proto-API-v2
 date: Feb 2014

All features marked with [A] already existed in Python-Arduino-Proto-API-v2
project, but most of them have been modified.

Copyright (C) 2014  Álvaro Muñoz

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or any 
later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see http://www.gnu.org/licenses/ .

    Feel free to mail me if bugs or new ideas!
 
*/

#ifndef SERIAL_RATE
#define SERIAL_RATE         9600
#endif

#ifndef SERIAL_TIMEOUT
#define SERIAL_TIMEOUT      5
#endif

#define MAX_SERVOS          5

#include <Servo.h>

// Servo structure
typedef struct{
  Servo s;
  int pos;
  int pin;
} servo;

servo servos[MAX_SERVOS]; // Array of servos (max 5)
int nServ;       // Number of servos attached
int pinAux;      // Auxiliar pin 
int freq;        // Buzzer frequency tone
int duration;    // Duration of buzzer tone
int i;           // Index of servos array

void setup() {
    Serial.begin(SERIAL_RATE);
    Serial.setTimeout(SERIAL_TIMEOUT);
    
    // Read de number of OUTPUT set pins
    int cmd = readData();
    for (int i = 0; i < cmd; i++) {
        pinMode(readData(), OUTPUT);
    }
    
    // Actual servos are 0
    nServ = 0;
}

void loop() {
    switch (readData()) {
        case 0 :
            // [A] set digital low
            digitalWrite(readData(), LOW); break;
        case 1 :
            // [A] set digital high
            digitalWrite(readData(), HIGH); break;
        case 2 :
            // [A] get digital value
            Serial.println(digitalRead(readData())); break;
        case 3 :
            // [A] set analog value
            analogWrite(readData(), readData()); break;
        case 4 :
            // [A] read analog value
            Serial.println(analogRead(readData())); break;
        case 5:
            // move servo to specific angle (0-180º)
            // find the servo attached to the read pin
            pinAux=readData();
            for(i=0; i<nServ; i++)
                if(servos[i].pin==pinAux)
                  break;
            servos[i].pos=readData();
            // constrain and reverse the angles
            servos[i].pos=map(constrain(servos[i].pos,0,179),0,179,179,0);
            servos[i].s.write(servos[i].pos);
            delay(100);
            break;
        case 6:
            // set a servo pin and put it on 90º
            if(nServ<MAX_SERVOS){
              servos[nServ].pin = readData();
              servos[nServ].s.attach(servos[nServ].pin);
              servos[nServ].pos=90;
              servos[nServ].s.write(servos[nServ].pos);
              nServ++;
            }
            else readData();
            delay(30);
            break;
        case 7:
            // makes a beep with a frequency and duration 
            pinAux = readData();
            freq = readData();
            duration = readData();
            if(freq>0 && duration>0){
              tone(pinAux,freq,duration);
              delay(10);
              break;
            }
        case 99:
            // [A] just to cancel the current read, needed to prevent lock 
            // when the PC side dropped the "w" that we sent
            break;
    }
}

// [A] Read data from serial port
int readData() {
    Serial.println("w");
    while(1) {
        if(Serial.available() > 0) 
            return  Serial.parseInt();
    }
}
